import './Tooltip.scss';
import ParameterValidator from '@mvps-genesis/parameter-validator';

/*
 *--------------------------------------------------------------------------
 * Tooltip
 *--------------------------------------------------------------------------
 *
 * Class to enable creation and interaction with tooltips.
 */
export default class Tooltip {

	/**
	 * Parameter Validator object
	 *
	 * @var	@mvps-genesis/parameter-validator	ParameterValidator
	 */
	#validator;

	/**
	 * The tooltip element
	 *
	 * @var	Element
	 */
	element;

	/**
	 * The tooltip label element
	 *
	 * @var	Element
	 */
	labelElement;

	/**
	 * The tooltip drop down element
	 *
	 * @var	Element
	 */
	dropdownElement;

	/**
	 * Tooltip open class name
	 *
	 * @var	String
	 */
	openClass;

	/**
	 * Create a new Tooltip object
	 *
	 * @param  {HTMLElement|String}  tooltipElement           The tooltip element.
	 * @param  {HTMLElement|String}  options.dropdownElement  The dropdown element. Defaults to '.g-tooltip__dropdown'.
	 * @param  {HTMLElement|String}  options.labelElement     The label element. Defaults to '.g-tooltip__label'.
	 * @param  {String}              options.openClass        The tooltip open class. Defaults to 'g-tooltip--open'.
	 */
	constructor (tooltipElement, {
		dropdownElement = '.g-tooltip__dropdown',
		labelElement = '.g-tooltip__label',
		openClass = 'g-tooltip--open'
	} = {}) {
		// Create param validator
		this.#validator = new ParameterValidator(this);

		this.element = this.#validator.isElement('tooltipElement', tooltipElement);

		this.labelElement = this.#validator.isElement('labelElement', labelElement, {container: this.element});
		this.dropdownElement = this.#validator.isElement('dropdownElement', dropdownElement, {container: this.element});
		this.openClass = this.#validator.isString('openClass', openClass);

		this.bindEvents();
	}

	/**
	 * Bind the base events
	 */
	bindEvents () {
		this.labelElement.addEventListener('mouseover', () => this.handleDropdownPosition());
		this.labelElement.addEventListener('touchstart', () => this.toggle());

		document.addEventListener('touchstart', (event) => {
			if (typeof event.target.closest !== 'function') {
				return;
			}

			const currentTooltip = event.target.closest('.js-tooltip');
			const openTooltips = document.getElementsByClassName(this.openClass);

			for (const tooltip of openTooltips) {
				if (tooltip === currentTooltip) {
					return;
				}

				tooltip.classList.remove(this.openClass);
			}
		});
	}

	/**
	 * Handle edge detection on mobile devices for horizontal drop down position
	 */
	handleDropdownPosition () {
		const screenPadding = 8;
		const labelRect = this.labelElement.getBoundingClientRect();
		const dropdownRect = this.dropdownElement.getBoundingClientRect();
		const labelRightX = labelRect.x + labelRect.width;
		const dropdownRightX = dropdownRect.x + dropdownRect.width;

		if (dropdownRect.x < 0) {
			this.dropdownElement.style.left = '0';
			this.dropdownElement.style.transform = `translateX(${-labelRect.x + screenPadding}px)`;
		} else if (dropdownRightX > window.outerWidth) {
			this.dropdownElement.style.right = '0';
			this.dropdownElement.style.transform = `translateX(${window.outerWidth - labelRightX - screenPadding}px)`;
		}
	}

	toggle () {
		if (this.element.classList.contains(this.openClass)) {
			this.close();
		} else {
			this.open();
		}
	}

	open () {
		this.element.classList.add(this.openClass);
		this.handleDropdownPosition();
	}

	close () {
		this.element.classList.remove(this.openClass);
	}
}
